/*
  Airsoft Hit Detection System, subproject of "Open Robotics" project
  
 The circuit:
 * pin 4 connected to HIT led
 * pin 7 controls tank power (LOW - power off, HIGH - power on)
 * pin 6 controls speaker (for "hit", "game over", and "restart" sounds)
 * pin 10 connected to "restart" button
 * pins A0..A5 connected to piezo-sensors
 
 Created 27 November 2010
 By Anton A. Botov,
 
 http://open-robotics.ru/
 http://roboforum.ru/
 
 */

//Game parameters
int maxhits = 3;
int adc_threshould = 100;

//Game variables
int countdown=0;
int hits=0;

//Scheme connections
int sensor0 = 0;
int ledPin = 4;
int lampPin = 9;
int tankPin = 7;
int btnPin = 10;
int spkPin = 6;

void hit_sound()
{
  for(int i=0; i<150; i++){
    int j=(i*127)%157;
    j=500+j*20;
    tone(spkPin,j,2);
    delay(2);
  };
};

void destroy_sound()
{
  hit_sound();
  hit_sound();
  hit_sound();
  
  delay(150);
  
  for(int16_t v=150; v>30; v-=2){ tone(spkPin,500+v*10,6); delay(6); };
};

void restart_sound()
{
  for(int v=30; v<150; v++){ tone(spkPin,500+v*10,2); delay(2); };
  for(int v=150; v>30; v--){ tone(spkPin,500+v*10,2); delay(2); };
};


void setup()
{
  //led
  pinMode(ledPin,OUTPUT);
  pinMode(lampPin,OUTPUT);
  
  //tank power control
  pinMode(tankPin,OUTPUT);
  digitalWrite(tankPin,LOW); //Switch-on tank power

  //Button input
  pinMode(btnPin,INPUT);
  digitalWrite(btnPin,HIGH); //activate pull-up for btn
}

void loop()                     
{
  
  //Show hit by led
  if(countdown<200){
    digitalWrite(ledPin,LOW);
    digitalWrite(lampPin,LOW);
  }else{
    digitalWrite(ledPin,HIGH);
    digitalWrite(lampPin,HIGH);
  };

  int lasthits=hits;

  if(
    (analogRead(0)>adc_threshould) ||
    (analogRead(1)>adc_threshould) ||
    (analogRead(2)>adc_threshould) ||
    (analogRead(3)>adc_threshould) ||
    (analogRead(4)>adc_threshould) ||
    (analogRead(5)>adc_threshould)
  ){
    if(countdown==0){
	  hits++;
	  countdown=2000;
	};
  };

  if(lasthits<hits && lasthits<maxhits){
    if(hits==maxhits){
      digitalWrite(ledPin,HIGH);
      digitalWrite(lampPin,HIGH);
      destroy_sound();
    }else{
      digitalWrite(ledPin,HIGH);
      digitalWrite(lampPin,HIGH);
      hit_sound();
    };
  };
  
  if(hits>maxhits) hits=maxhits;

  //If button pressed
  if(digitalRead(btnPin)==LOW){
    restart_sound();
    countdown=190;
    hits=0;
  };

  if(hits==maxhits || countdown>1200){
    digitalWrite(tankPin,HIGH); //Switch-off tank power
  }else{
    digitalWrite(tankPin,LOW); //Switch-on tank power
  };

  if(countdown>0){
    countdown--;
    delay(1);
  };
  
  if(countdown==0 && hits>=maxhits){
    countdown=400;
  };
  
}
